/*
 * Program memory segment support.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/re/c/dataspace.h>
#include <l4/re/c/rm.h>

#include <fsclient/client.h>

#include <exec/common.h>
#include <exec/mapped_region.h>



/* Program segment abstractions. */

class Segment
{
protected:
  MappedRegion _region;
  struct exec_region _exec_region;

  /* Allocated memory. */

  char *_buf;
  l4re_ds_t _ds;
  file_t *_file;
  bool _allocated = false, _attached = false;

  /* Segment region base. */

  l4_addr_t _region_base;

  /* Segment region size. */

  offset_t _region_allocated_size;

  /* Offset of segment content within the region. */

  offset_t _region_content_offset;

  /* Common initialisation. */

  void init();

public:
  virtual ~Segment();

  l4_cap_idx_t cap();

  char *address();

  offset_t size();

  /* Segment population methods. */

  long allocate(bool attach);

  long fill(file_t *file, bool attach);

  /* Mapped region methods. */

  MappedRegion &region();

  l4_addr_t region_address(char *address);

  l4_addr_t region_address(l4_addr_t address);

  /* Generic property access. */

  struct exec_region &exec_region();

  virtual bool loadable() = 0;
  virtual offset_t file_contents() = 0;
  virtual offset_t file_offset() = 0;
  virtual l4_addr_t region_address() = 0;
  virtual offset_t region_size() = 0;
  virtual l4re_rm_flags_t region_flags() = 0;
};



/* An explicitly created segment independent of any particular payload. */

class ExplicitSegment : public Segment
{
protected:
  /* Segment base. */

  l4_addr_t _base;

  /* Segment size. */

  offset_t _size;

  /* Access flags. */

  l4re_rm_flags_t _flags;

  /* File access details. */

  offset_t _file_offset, _file_contents;

public:
  explicit ExplicitSegment(l4_addr_t base, offset_t size, l4re_rm_flags_t flags,
                           offset_t file_offset = 0, offset_t file_contents = 0);

  /* Generic property access. */

  bool loadable();
  offset_t file_contents();
  offset_t file_offset();
  l4_addr_t region_address();
  offset_t region_size();
  l4re_rm_flags_t region_flags();
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
